package eu.etaxonomy.taxeditor.editor.view.checklist.listener;

import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.ToolItem;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.taxeditor.editor.view.checklist.ChecklistEditor;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.wizard.AvailableDistributionWizard;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * Handles drop down menu selection. Available items are defined in the
 * enumeration SearchOption.
 *
 * @author n.hoffmann
 * @created Feb 2, 2010
 * @version 1.0
 */
public class ChecklistDropdownSelectionListener extends SelectionAdapter {

    private final Menu menu;
    private final ChecklistEditor editor;
    private final SortedSet<DefinedTermBase> termSet;
    private Logger logger = Logger.getLogger(ChecklistDropdownSelectionListener.class);
    
    public ChecklistDropdownSelectionListener(ToolItem dropdown, ChecklistEditor editor,
            SortedSet<DefinedTermBase> termSet) {
        this.editor = editor;
        this.termSet = termSet;
        menu = new Menu(dropdown.getParent().getShell());
        initialMenuItem();
    }

    // SearchOption option
    public void add(DefinedTermBase<DefinedTermBase> term) {
        MenuItem menuItem = new MenuItem(menu, SWT.CHECK);
        menuItem.setText(term.getTitleCache());
        if (PreferencesUtil.getPreferenceStore().getBoolean(term.getUuid().toString())) {
            menuItem.setSelection(true);
        } else {
            menuItem.setSelection(false);
        }
        menuItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                MenuItem selected = (MenuItem) event.widget;

                TableViewer viewer = (TableViewer) editor.getViewer();
                final Table table = viewer.getTable();
                // TODO make it selectable according to selection event
                TableColumn column = null;
                DefinedTermBase<DefinedTermBase> selectedTerm = null;
                if (termSet != null) {
                	TableColumn[] columns = viewer.getTable().getColumns();
                	for(int i=0; i<columns.length; i++){
                		if(selected.getText().equalsIgnoreCase(columns[i].getText())){
                			column = columns[i];
                			logger.info("Column no " +i +" Column Header "+ column.getText() );
                			for (DefinedTermBase<DefinedTermBase> term : termSet) {
                				if(column.getText().equalsIgnoreCase(term.getTitleCache())){
                					selectedTerm = term;
                					break;
                				}
                			}
                		}
                	}
                }
                if (column != null && column.getWidth() == 0) {
                    // save column status if shown or not
                    if (selectedTerm != null) {
                        PreferencesUtil.getPreferenceStore().setValue(selectedTerm.getUuid().toString(), true);
                    }
                    column.setWidth(100);
                } else if (column != null) {
                    if (selectedTerm != null) {
                        PreferencesUtil.getPreferenceStore().setValue(selectedTerm.getUuid().toString(), false);
                    }
                    column.setWidth(0);
                }
            }
        });
    }

    private void initialMenuItem(){
        MenuItem menuItem = new MenuItem(menu, SWT.PUSH);
        menuItem.setText("Add Distribution");
        menuItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                AvailableDistributionWizard availableDistributionWizard = new AvailableDistributionWizard();
                WizardDialog dialog = new WizardDialog(StoreUtil.getShell(),
                        availableDistributionWizard);

                int open = dialog.open();
                if(open == 0){
                	editor.reload();
                }
            }
        });
        new MenuItem(menu, SWT.SEPARATOR);
    }

    @Override
    public void widgetSelected(SelectionEvent event) {
        if (event.detail == SWT.ARROW) {
            ToolItem item = (ToolItem) event.widget;
            Rectangle rect = item.getBounds();
            Point pt = item.getParent().toDisplay(new Point(rect.x, rect.y));
            menu.setLocation(pt.x, pt.y + rect.height);
            menu.setVisible(true);
        } else {
            menu.setVisible(true);
        }
    }
    
    public Menu getMenu(){
    	return menu;
    }
}