// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.http.client.ClientProtocolException;

import eu.etaxonomy.cdm.ext.occurrence.bioCase.BioCaseQueryServiceWrapper;
import eu.etaxonomy.cdm.io.common.CdmDefaultImport;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.view.dataimport.transientServices.TransientCdmRepository;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date Sep 3, 2014
 *
 */
public class SpecimenImportView extends DataImportView<SpecimenOrObservationBase<?>> {

    public static final String ID = "eu.etaxonomy.taxeditor.editor.view.dataimport.SpecimenImportView"; //$NON-NLS-1$

    private URI endPoint;

    public SpecimenImportView() {
    }

    /**
     * @param endPoint
     */
    public void setEndpoint(URI endPoint) {
        this.endPoint = endPoint;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.view.dataimport.DataImportView#getTextForTableItem(java.lang.Object)
     */
    @Override
    protected String getTextForTableItem(SpecimenOrObservationBase<?> item) {
        return item.getTitleCache();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.view.dataimport.DataImportView#query()
     */
    @Override
    public void query() {
        String errorMessage = "Could not execute query " + query;

        Collection<SpecimenOrObservationBase<?>> results = new ArrayList<SpecimenOrObservationBase<?>>();
        try {
            //FIXME move ABCD import to cdmlib -> this will also get rid of the transient services
            InputStream resultStream;
            resultStream = new BioCaseQueryServiceWrapper().query(query, endPoint);
            Abcd206ImportConfigurator configurator = Abcd206ImportConfigurator.NewInstance(resultStream, null, false);
            TransientCdmRepository repo =
                    new TransientCdmRepository(CdmStore.getCurrentApplicationConfiguration());
            configurator.setCdmAppController(repo);
            configurator.setAddMediaAsMediaSpecimen(true);

            CdmDefaultImport<Abcd206ImportConfigurator> importer = new CdmDefaultImport<Abcd206ImportConfigurator>();
            importer.invoke(configurator);
            results = repo.getUnits();
            setResults(results);
        } catch (ClientProtocolException e) {
            logger.error(errorMessage, e);
        } catch (IOException e) {
            logger.error(errorMessage, e);
        }

        setResults(results);

    }

}
