// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import junit.framework.Assert;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToConceptOperation;
import eu.etaxonomy.taxeditor.store.operations.AbstractTaxeditorOperationTestBase;

/**
 * @author n.hoffmann
 * @created 07.04.2009
 * @version 1.0
 */
public class ChangeSynonymToConceptOperationTest extends AbstractTaxeditorOperationTestBase{
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToConceptOperationTest.class);

	private static TaxonRelationshipType taxonRelationshipType;

	private static Synonym synonym;

	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		
		taxon = Taxon.NewInstance(null, null);
		synonym = Synonym.NewInstance(NonViralName.NewInstance(null), null);
		taxonRelationshipType = TaxonRelationshipType.CONTRADICTION();
		
		taxon.addSynonym(synonym, SynonymRelationshipType.SYNONYM_OF());
		
		operation = new ChangeSynonymToConceptOperation("Change Synonym To Concept", null, taxon, synonym, taxonRelationshipType, postOperation);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToConceptOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecuteIProgressMonitorIAdaptable() throws ExecutionException {
		operation.execute(monitor, info);
		
		Assert.assertTrue("Synonym should be deleted from taxon.", taxon.getSynonyms().size() == 0);
		Assert.assertTrue("Taxon should have taxon relationships now.", taxon.getRelationsToThisTaxon().size() > 0);
		Assert.assertEquals("Relationship is not the expected", taxonRelationshipType, taxon.getTaxonRelations().toArray(new TaxonRelationship[0])[0].getType());
		Assert.assertEquals("Name of synonym and newly created related taxon should be equal", synonym.getName(), taxon.getTaxonRelations().toArray(new TaxonRelationship[0])[0].getFromTaxon().getName());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToConceptOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.undo(monitor, info);
		
		Assert.assertTrue("There should be synonyms now.", taxon.getSynonyms().size() > 0);
		Assert.assertEquals("Synonym is not the expected.", synonym, taxon.getSynonyms().toArray(new Synonym[0]));
		Assert.assertTrue("There should not be taxon relationships.", taxon.getRelationsToThisTaxon().size() == 0);
	}
	
	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.ChangeSynonymToConceptOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.redo(monitor, info);
				
		Assert.assertTrue("Synonym should be deleted from taxon.", taxon.getSynonyms().size() == 0);
		Assert.assertTrue("Taxon should have taxon relationships now.", taxon.getRelationsToThisTaxon().size() > 0);
		Assert.assertEquals("Relationship is not the expected", taxonRelationshipType, taxon.getTaxonRelations().toArray(new TaxonRelationship[0])[0].getType());
		Assert.assertEquals("Name of synonym and newly created related taxon should be equal", synonym.getName(), taxon.getTaxonRelations().toArray(new TaxonRelationship[0])[0].getFromTaxon().getName());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation#postExecute(ICdmBase)}.
	 */
	@Test
	public void testPostExecute() {
		// TODO there is not post operation functionality for this class
		// at the moment. Implement test when there is.
		logger.warn("No post operation functionality for this class");
	}
}
